@@ -1,6 +1,8 @@
 import copy
 import functools
+import hashlib
 import operator
+import pickle
 import random
 from abc import ABC, abstractmethod
 from dataclasses import asdict, dataclass
@@ -56,7 +58,19 @@ VectorT = TypeVar("VectorT", bound="VectorBase")
 CompositeListElementT = TypeVar("CompositeListElementT")
 
 
-class HasAList(Sequence, Generic[CompositeListElementT]):
+class HashableMixin:
+    @property
+    def hash_value(self) -> str:
+        data_pickle = pickle.dumps(self)
+        data_md5 = hashlib.md5(data_pickle).hexdigest()
+        return data_md5[:8]
+
+
+class MetaData(Dict[str, Union[str, int, float]], HashableMixin):
+    pass
+
+
+class HasAList(Generic[CompositeListElementT]):
     @abstractmethod
     def _get_has_a_list(self) -> List[CompositeListElementT]:
         pass
@@ -64,8 +78,24 @@ class HasAList(Sequence, Generic[CompositeListElementT]):
     def __iter__(self) -> Iterator[CompositeListElementT]:
         return self._get_has_a_list().__iter__()
 
+    @overload
+    def __getitem__(self, indices: List[int]) -> List[CompositeListElementT]:
+        pass
+
+    @overload
+    def __getitem__(self, indices: slice) -> List[CompositeListElementT]:
+        pass
+
+    @overload
+    def __getitem__(self, index: int) -> CompositeListElementT:
+        pass
+
     def __getitem__(self, indices_like):  # TODO(HiroIshida) add type hints?
-        return self._get_has_a_list()[indices_like]
+        lst_inner = self._get_has_a_list()
+        if isinstance(indices_like, list):
+            return [lst_inner[i] for i in indices_like]
+        else:
+            return lst_inner[indices_like]
 
     def __len__(self) -> int:
         return len(self._get_has_a_list())
@@ -122,6 +152,12 @@ class PrimitiveElementBase(ElementBase):
     def __getitem__(self, index):
         return self._data[index]
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, PrimitiveElementBase):
+            return NotImplemented
+        assert type(self) == type(other)
+        return np.allclose(self._data, other._data, atol=1e-6)
+
 
 class VectorBase(PrimitiveElementBase):
     def __init__(self, data: np.ndarray) -> None:
@@ -379,6 +415,15 @@ class CompositeImageBase(ImageBase):
         for image in self.images:
             image.resize(shape2d_new)
 
+    def __eq__(self, other: object) -> bool:
+        if not isinstance(other, CompositeImageBase):
+            return NotImplemented
+        assert type(self) == type(other)
+        for im_self, im_other in zip(self.images, other.images):
+            if im_self != im_other:
+                return False
+        return True
+
 
 class RGBDImage(CompositeImageBase):
     image_types = [RGBImage, DepthImage]
@@ -414,6 +459,15 @@ class ElementDict(Dict[Type[ElementBase], ElementBase]):
         else:
             assert False
 
+    def __eq__(self, other: object):
+        if not isinstance(other, ElementDict):
+            return NotImplemented
+        assert isinstance(other, ElementDict)
+        for key in self.keys():
+            if self[key] != other[key]:
+                return False
+        return True
+
 
 def get_element_type(type_name: str) -> Type[ElementBase]:
     for t in get_all_concrete_leaftypes(ElementBase):
@@ -468,10 +522,30 @@ class TypeShapeTableMixin:
         return list(self.type_shape_table.keys())  # type: ignore
 
 
+@dataclass
+class TimeStampSequence(HasAList[float]):
+    """data must be increasing order"""
+
+    _data: List[float]
+
+    def __post_init__(self):
+        assert sorted(self._data) == self._data
+
+    def _get_has_a_list(self) -> List[float]:
+        return self._data
+
+    def index_geq(self, time: float) -> int:
+        """find index greater than or equal to time"""
+        eps = 1e-8
+        return [i for i, t in enumerate(self._data) if t > time - eps][0]
+
+
 @dataclass(frozen=True)
 class EpisodeData(TypeShapeTableMixin):
     sequence_dict: Dict[Type[ElementBase], ElementSequence[ElementBase]]
     type_shape_table: Dict[Type[ElementBase], Tuple[int, ...]]
+    time_stamp_seq: Optional[TimeStampSequence] = None
+    metadata: Optional[MetaData] = None
 
     def __post_init__(self):
         ef_seq = self.get_sequence_by_type(TerminateFlag)
@@ -504,13 +578,20 @@ class EpisodeData(TypeShapeTableMixin):
         assert change_count == 1
 
     @classmethod
-    def from_seq_list(cls, sequence_list: List[ElementSequence]):
+    def from_seq_list(
+        cls,
+        sequence_list: List[ElementSequence],
+        timestamp_seq: Optional[TimeStampSequence] = None,
+        metadata: Optional[MetaData] = None,
+    ):
 
         for sequence in sequence_list:
             assert isinstance(sequence, ElementSequence)
 
         all_same_length = len(set(map(len, sequence_list))) == 1
         assert all_same_length
+        if timestamp_seq is not None:
+            assert len(sequence_list[0]) == len(timestamp_seq)
 
         types = [type(seq[0]) for seq in sequence_list]
 
@@ -527,7 +608,7 @@ class EpisodeData(TypeShapeTableMixin):
         assert all_different_type, "all sequences must have different type"
 
         sequence_dict = {seq.elem_type: seq for seq in sequence_list}
-        return cls(sequence_dict, type_shape_table)  # type: ignore
+        return cls(sequence_dict, type_shape_table, timestamp_seq, metadata)  # type: ignore
 
     def get_sequence_by_type(self, elem_type: Type[ElementT]) -> ElementSequence[ElementT]:
 
@@ -543,6 +624,11 @@ class EpisodeData(TypeShapeTableMixin):
     def __getitem__(self, index: int) -> ElementDict:
         pass
 
+    @overload
+    def __getitem__(self, indices: List[int]) -> "EpisodeData":
+        """remove TerminateFlag"""
+        pass
+
     @overload
     def __getitem__(self, slicee: slice) -> "EpisodeData":
         """remove TerminateFlag"""
@@ -552,15 +638,64 @@ class EpisodeData(TypeShapeTableMixin):
         if isinstance(index_like, int):
             elems = [seq[index_like] for seq in self.sequence_dict.values()]
             return ElementDict(elems)
-        elif isinstance(index_like, slice):
+        elif isinstance(index_like, slice) or isinstance(index_like, list):
+
+            if self.time_stamp_seq is None:
+                partial_ts_seq = None
+            else:
+                partial_ts_seq = TimeStampSequence(self.time_stamp_seq[index_like])
+
             partial_seq_list = []
             for seq in self.sequence_dict.values():
                 if seq.elem_type != TerminateFlag:
+                    # TODO(HiroIshida): remove type ignore
                     partial_seq_list.append(ElementSequence(seq[index_like]))  # type: ignore
-            return EpisodeData.from_seq_list(partial_seq_list)
+            return EpisodeData.from_seq_list(partial_seq_list, timestamp_seq=partial_ts_seq)
         else:
             assert False
 
+    def slice_by_time(
+        self, t_start_data: float, t_end_data: float, t_end_task: Optional[float] = None
+    ) -> "EpisodeData":
+        """slice episode using times.
+        t_start_data: start time of data
+        t_end_data: end time of this episode
+        t_end_task: input this if t_end_task does not match t_end_data. This is important if TerminateFlag before the terminal index.
+
+        These three values must satisfy following realtions t_start_data < t_end_task < t_end_data
+
+        """
+        assert t_start_data < t_end_data
+
+        assert self.time_stamp_seq is not None
+        i_start_data = self.time_stamp_seq.index_geq(t_start_data)
+        i_end_data = self.time_stamp_seq.index_geq(t_end_data)
+
+        assert i_start_data < i_end_data
+        partial_episode = self[i_start_data : i_end_data + 1]
+
+        if t_end_task is None:
+            return partial_episode
+        else:
+            assert t_start_data < t_end_task < t_end_data
+            # create flag sequence considering t_end_task
+            i_end_task = self.time_stamp_seq.index_geq(t_end_task)
+            flag_list_false = [
+                TerminateFlag.from_bool(False) for _ in range(i_end_task - i_start_data)
+            ]
+            flag_list_true = [
+                TerminateFlag.from_bool(True) for _ in range(i_end_data - i_end_task + 1)
+            ]
+            flag_seq = ElementSequence[TerminateFlag](flag_list_false + flag_list_true)
+
+            # bit dirty...
+            assert partial_episode.time_stamp_seq is not None  # just for mypy
+            time_seq = partial_episode.time_stamp_seq
+            seq_dict = copy.deepcopy(partial_episode.sequence_dict)
+            # TODO(HiroIshida) remove type-ignore (generic dict..?)
+            seq_dict[TerminateFlag] = flag_seq  # type: ignore
+            return EpisodeData.from_seq_list(list(seq_dict.values()), timestamp_seq=time_seq)
+
     def save_debug_gif(self, filename: str, fps: int = 20):
         t: Type[ImageBase]
         if RGBDImage in self.types():
@@ -577,16 +712,13 @@ class EpisodeData(TypeShapeTableMixin):
         clip.write_gif(filename, fps=fps)
 
 
-ExtraInfoType = Optional[Dict[str, Union[str, int, float]]]
-
-
 @dataclass(frozen=True)
 class ChunkSpec(TypeShapeTableMixin):
     n_episode: int
     n_episode_intact: int
     n_average: int
     type_shape_table: Dict[Type[ElementBase], Tuple[int, ...]]
-    extra_info: ExtraInfoType = None
+    extra_info: Optional[MetaData] = None
 
     def to_dict(self) -> Dict:
         d = asdict(self)
@@ -621,7 +753,7 @@ class MultiEpisodeChunk(HasAList[EpisodeData], TypeShapeTableMixin):
     def from_data_list(
         cls,
         data_list: List[EpisodeData],
-        extra_info: ExtraInfoType = None,
+        extra_info: Optional[MetaData] = None,
         shuffle: bool = True,
         with_intact_data: bool = True,
     ) -> "MultiEpisodeChunk":
@@ -641,8 +773,15 @@ class MultiEpisodeChunk(HasAList[EpisodeData], TypeShapeTableMixin):
             for idx in sorted(indices_intact, reverse=True):
                 data_list_intact.append(data_list.pop(idx))
 
+        # use fixed random seed where it's scope is only in this function
+        # The reason why I used fixed seed is to keep two different shuffled
+        # chunk generated has the contents in the same order.
+        # This is particulary important say you make a chunk with 5hz and another
+        # with 20hz, and do some computation using both of chunks.
+        rn = random.Random()
+        rn.seed(0)
         if shuffle:
-            random.shuffle(data_list)
+            rn.shuffle(data_list)
 
         type_shape_table = data_list[0].type_shape_table
 
@@ -705,44 +844,18 @@ class MultiEpisodeChunk(HasAList[EpisodeData], TypeShapeTableMixin):
     def get_not_intact_chunk(self) -> "MultiEpisodeChunk":
         return MultiEpisodeChunk(self.data_list, [], self.type_shape_table, self.spec)
 
-    def merge(self, other: "MultiEpisodeChunk") -> None:
-        keys_self = set(self.types())
-        keys_other = set(other.types())
-        assert keys_other.issubset(
-            keys_self
-        )  # TODO(HiroIshida) current limitation, and easily remove this assertion
-        keys_common = keys_self.intersection(keys_other)
-
-        def filter_episode_data_list(episode_data_list: List[EpisodeData]):
-            # TODO(HiroIshida) not efficient at all...
-            episode_data_list_filtered = []
-            for episode_data in episode_data_list:
-                seqs = []
-                for key in keys_common:
-                    seqs.append(episode_data.get_sequence_by_type(key))
-                episode_data_list_filtered.append(EpisodeData.from_seq_list(seqs))
-            assert len(episode_data_list) == len(episode_data_list_filtered)
-            return episode_data_list_filtered
-
-        data_list_new = other.data_list
-        data_list_intact_new = other.data_list_intact
-        data_list_new.extend(filter_episode_data_list(self.data_list))
-        self.data_list = data_list_new
-        self.data_list_intact = data_list_intact_new
-        self.type_shape_table = other.type_shape_table
-
     def plot_vector_histories(
         self,
         elem_type: Type[VectorBase],
         project_name: Optional[str] = None,
         hz: Optional[float] = None,
+        postfix: Optional[str] = None,
     ) -> None:
 
         n_vec_dim = self.spec.type_shape_table[elem_type][0]
 
         fig = plt.figure()
         fig, axs = plt.subplots(n_vec_dim)
-
         for i_dim, ax in enumerate(axs):
 
             y_min, y_max = +np.inf, -np.inf  # will be updated in the following loop
@@ -772,7 +885,10 @@ class MultiEpisodeChunk(HasAList[EpisodeData], TypeShapeTableMixin):
             axs[-1].set_xlabel("time [s]")
 
         project_path = get_project_path(project_name)
-        file_path = project_path / "seq-{}.png".format(elem_type.__name__)
+        if postfix is None:
+            file_path = project_path / "seq-{0}.png".format(elem_type.__name__)
+        else:
+            file_path = project_path / "seq-{0}-{1}.png".format(postfix, elem_type.__name__)
         file_name = str(file_path)
         fig.savefig(file_name, format="png", dpi=300)
         print("saved to {}".format(file_name))
